classdef AMS3536 <hgsetget
    %AMS3536 handle Class
    %   a class to interface with A-M Systems Model 3500 and 3600
    %   Instrument Amplifiers.
    %   Detailed explanation goes here
    
    %% Properties
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    properties(Constant)
      lblChannelColumns = '[Ch# Mode HP Notch LP Gain Ref]';
      lblGlobalColums='[StimSource CalAmp CalOn RefSource MonA MonB]';
    end
    properties (Dependent)
    Channels
    Globals
    end
    properties (Access=private)
    priv_Channels=[(1:16)' zeros(16,6,'uint8')];
    priv_Globals=[0 0 0 0 0 0];
    priv_InstrumentName='';
    bRef_2to8=0;
    bRef_10to16=0;
    bNotch_2to8=0;
    bNotch_10to16=0;
    bRefNotch_1and9=0;   
    PortInfo;
    end   
    properties (SetAccess='protected', GetAccess='public')
        ComPort='Nan'; 
        Control='Disconnected';
        PortSuccess=0;
        ProgramNumber=0;
        SerialNumber='';
        FirmWare=0;
        Protocol=0;
        HardWareConfig=zeros(1,320);  
        InstrumentName='';
        FlashNames={'' '' '' '' '' };
    end
    properties (Access='protected', Constant=true)
        msgReadProtocolVersion = hex2dec('A0');
        msgReadSerialNumber = hex2dec('A2');
        msgReadFirmwareVersion = hex2dec('A4');
        msgReadInstrumentName = hex2dec('A6');
        msgReadHardwareConfig = hex2dec('AA');
        msgReadActiveProgram = hex2dec('B0');
        msgReadFlashBlock  = hex2dec('B1');
        msgLoadFlashBlock  = hex2dec('B2');
        msgSaveActiveProgramInFlash = hex2dec('B3');
        msgWriteFlashProgram = hex2dec('B4');
        msgWriteInstrumentName = hex2dec('AC');
        msgWriteSettingValue  = hex2dec('B5');
        msgWriteProgram = hex2dec('B6');
        msgReadFlashNames  = hex2dec('B7');
        msgTakeControl = hex2dec('B9');
        msgReadStatus = hex2dec('BA');

        setMode =2;
        setHP =3;
        setNotch =4;
        setLP =5;
        setGain =6;
        setRef =7;
    end
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    
%%  Public Methods
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
methods 
    %%  Constructor Method
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    function thisAMS =AMS3536(amsPort)
    % AMS3536 - Constructor for the class
    %
    %   thisAMS = AMS3536(amsPort) creates a handle to an
    %   instance of the AMS3536 Class. The instance is set to be
    %   associated with the Instrument that is paired with the com port
    %   defined in the input amsPort.
    %
    %   SYNOPSIS: thisAMS =AMS3536(amsPort)
    %
    %   INPUT: amsPort - string value defining the numeric value of the
    %                    desired Com Port
    %
    %   OUTPUT: thisAMS - handle to the instance of the
    %                         AMS3536 Class
    %
    %   EXAMPLE: Model3500 = AMS3536('COM7');
    %
        if nargin == 0
            thisAMS.ComPort='COM6';
        else
            thisAMS.ComPort=amsPort;
        end
    end % constructor
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    %% Channels Property 
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    %
    % The Channels property holds the settings for each individual channel
    % Settings are shown below for standard settings or read from
    % the Hardware Configuration Block for custom instruments.
    % Channels is a 16x7 matrix with the following collums:
    %   [Ch#    Mode    HP  Notch   LP  Gain    Ref]
    %
    % Valid values are:
    %   Ch#     =   1 to 16
    %   Mode    =   0 (off), 1 (on), 2 (stim)
    %   HP      =   0 to 7. 0=lowest frequncy, 7=highest frequency
    %      Standard values: 0.3Hz 1Hz 3Hz 10Hz 30Hz 100Hz 300Hz 500Hz
    %   Notch   =   0 (Off), 1 (on)
    %   LP      =   0 to 7. 0=lowest frequncy, 7=highest frequency
    %      Standard values: 100Hz 300Hz 500Hz 1kHz 3kHz 5kHz 10kHz 20kHz 
    %   Gain    =   0 to 12 0=lowest gain, 12=highest gain
    %      Standard values: 2 4 10 20 50 100 200 500 1k 2k 5k 10k 20k
    %   Ref     =   0 (ref or GND), 1 (Bus)
    
    %   EXAMPLE: AMS1.Channels(3,:)=[3 0 2 0 7 2 0];  
    %         or AMS1.Channels(3,4)=7;
    %
    function out = get.Channels(thisAMS)
        out=thisAMS.priv_Channels;
    end
    function thisAMS=set.Channels(thisAMS,value)
        if size(value) ~= [16 7]
             display('You must define a value or range of values of Channels.');
             display('For example AMS3536.Channels(3,4)=7');
            error('Or, Channels(3,:)=[3 0 2 0 7 2 0]');
        else
            if max(max(value(:,2:7))) >12 || min(min(value(:,2:7)))<0
                error('Values must be 0 to 7');
            else
                [r,c,v]=find(value ~= thisAMS.Channels);
                if isempty(find(c==1))
                    if min(size(r))~=0 
                        [errmsg,outBytes]=checkValue(thisAMS,r,c,diag(value(r,c)));
                        if strcmp(errmsg,'OK')
                            if mean(ismember(1:16,r))==1
                                bytes= convertdata(thisAMS,value, thisAMS.Globals);
                                obytes=SendReceive(thisAMS,thisAMS.msgWriteProgram,bytes);
                                thisAMS.PortSuccess=isequal(obytes(5:end-1),bytes);
                            else 
                                for i=1:size(outBytes,1)
                                    thisAMS.PortSuccess = WriteValue(thisAMS, outBytes(i,:)');
                                end
                            end
                            thisAMS.priv_Channels=value;
                        else
                            error(errmsg);
                        end
                    end
                else
                  error('The first collumn must be the channel number, and match the matrix index. E.G. AMS3536.Channels(4,:)=[4 0 0 0 0 0 0],or AMS3536.Channels(4,2)=1')
                end
            end
        end  
    end  
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    %%  Globals Property 
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% 
    % The Globals property are the settings that are the same for all
    % channels.
    % Settings are shown below for standard settings or read from
    % the Hardware Configuration Block for custom instruments.
    % Globals is a 1x6 matrix with the following collums:
    %   [StimSource CalAmp CalOn RefSource MonA MonB]
    %
    % Valid values are:
    %   StimSource  =  3600: 0(Stim1), 1(Stim2). 3500: 0(Stim2) 1(Joined)
    %   CalAmp      =   0 to 3 = 1V to 1mVp-p
    %   CalOn       =   0 (Off), 1 (on)
    %   RefSource   =  3500: 0(ExtBNC), 1(GND). 3600: 1 to 17=Ch1-Ch16,Ref 
    %   MonA        =  1 to 16 = Ch1 to Ch16
    %   MonB        =  1 to 16 = Ch1 to Ch16
    %
    %   EXAMPLE: AMS1.Globals =[0 2 0 1 1 14];  
    %         or AMS1.Globals(5)=7;
    %
    
    function out = get.Globals(thisAMS)
        out=thisAMS.priv_Globals;
    end
    function thisAMS=set.Globals(thisAMS,value)
    %'[StimSource CalAmp CalOn RefSource MonA MonB]';
     if size(value) ~= [1 6]
         display('You must define a value or range of values of Globals.');
         error('For example AMS3536.Globals(4)=7');
     else
         c=find(value ~= thisAMS.Globals);
         if min(size(c))~=0 
             [errmsg,outBytes]=checkGlobes(thisAMS,c,value(c));
                if strcmp(errmsg,'OK')
                    thisAMS.priv_Globals=value;
                    for i=1:size(outBytes,1)
                        thisAMS.PortSuccess = WriteValue(thisAMS, outBytes(i,:)');
                    end
                else
                    error(errmsg);
                end
         end
     end  
    end
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    %%  Action Methods
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% 
    function isSet=TakeControl(thisAMS)
        %TakeControl - Allow computer to send data to the instrument
        %
        %   TakeControl() removes control of the instrument from the 
        %   front panel and allows the instrument to receive  
        %   instructions from the computer.
        %
        %   SYNOPSIS: isSet = thisAMS.TakeControl()
        %
        %   OUTPUT: isSet - Boolean value which indicates if the
        %                         operation was successful or not 
        %                           (1=TRUE, 0=FALSE).
        %
        %   EXAMPLE:   ok = AMS1.TakeControl;
        %
        %   See also connect
        
        if (strcmp(thisAMS.Control,'Instrument'))
            a=SendReceive(thisAMS,thisAMS.msgTakeControl);
            if size(a,1)<3
                thisAMS.Control='Instrument';
                isSet=false;
            elseif a(3)==201
                thisAMS.Control='Computer';
                isSet=true;
            end
        else
            isSet = false;
            fprintf(strcat('Warning: Cannot take control on COM: ',thisAMS.ComPort,' AMS Instrument is not Connected.\n'))
        end
    end
    function isConnected = connect(thisAMS)
            %connect - Connect to the Instrument
            %
            %   connect() establishes a serial connection between the 
            %   instrument and computer using the com port defined 
            %   in the ComPort property of the object.
            %
            %   SYNOPSIS: isConnected = thisAMS.connect()
            %
            %   OUTPUT: isConnected - Boolean value which indicates if the
            %                         operation was successful or not 
            %                           (1=TRUE, 0=FALSE).
            %
            %   EXAMPLE:   ok = AMS1.connect;
            %
            %   See also TakeControl
        dataout=sscanf('A6 7F', '%x');
        if (strcmp(thisAMS.Control,'Computer') || strcmp(thisAMS.Control,'Instrument'))
            fprintf(strcat('Warning: AMS COM',thisAMS.ComPort,', is already connected.\n'));
            isConnected = true;
        elseif (strcmp(thisAMS.Control,'Disconnected'))
            try
                sp=serial(thisAMS.ComPort,'BaudRate',115200,'DataBits',8,'StopBits', 1);
                fopen(sp);
                fwrite(sp,dataout);
                pause(0.05);  % wait for transmission
                count=sp.BytesAvailable;
                if count > 3
                   fread(sp,count,'uint8');
                   isOpen=true;
                else
                   isOpen=false;
                   isConnected = false;
                   fprintf(strcat('Warning: Unable to communicate on COM: ',thisAMS.ComPort,' AMS Instrument is not Connected.\n'))
                end
                fclose(sp);
                if isOpen 
                    thisAMS.PortInfo=sp;
                    thisAMS.PortSuccess=true;
                    a=SendReceive(thisAMS,thisAMS.msgReadStatus);
                     if a(4) == 0
                         thisAMS.Control='Instrument';
                     else
                         thisAMS.Control='Computer';
                     end
                    [~,~,thisAMS.SerialNumber]=SendReceive(thisAMS,thisAMS.msgReadSerialNumber);
                    [~,~,thisAMS.InstrumentName]=SendReceive(thisAMS,thisAMS.msgReadInstrumentName);
                    [~,~,thisAMS.FlashNames]=SendReceive(thisAMS,thisAMS.msgReadFlashNames);
                    a=SendReceive(thisAMS,thisAMS.msgReadFirmwareVersion);
                    thisAMS.FirmWare=a(4:end-1)';
                    a=SendReceive(thisAMS,thisAMS.msgReadProtocolVersion);
                    thisAMS.Protocol=a(4)';  

                    a=SendReceive(thisAMS,thisAMS.msgReadHardwareConfig,[],3);
                    thisAMS.HardWareConfig=a(4:end-1)';
                    a=SendReceive(thisAMS,thisAMS.msgReadActiveProgram,[],.5);
                    thisAMS.ProgramNumber=a(4);
                    [thisAMS.priv_Channels, thisAMS.priv_Globals]=parseProgBlock(thisAMS,a(5:end-1));
                    [bRef_2to8, bRef_10to16, bNotch_2to8, bNotch_10to16, ...
                      bRefNotch_1and9]=updateBitVars(thisAMS,thisAMS.priv_Channels);
                end
                thisAMS.PortInfo=sp;        
            catch EX
                if strcmp(sp.Status,'open')
                    fclose(sp); 
                end
                isConnected = false;
                thisAMS.PortInfo=sp;
                thisAMS.PortSuccess=0;
                warning('No Instrument communication')
            end
        end
    end
    function [num, chandata, globdata]=ReadFlashBlock(thisAMS,block)
            %ReadFlashBlock - Read a saved setting set from the instrument.
            %
            %   ReadFlashBlock(n) Returns channel and global settings for
            %                     one of the five flash blocks.
            %
            %   SYNOPSIS: [n channels globals] = thisAMS.ReadFlashBlock(n)
            %
            %   INPUT: block - a number from 1 to 5 representing one of the
            %                  five complete instrument settings saved on
            %                  the instrument.
            %
            %   OUTPUT: n           -   the particular flash block
            %           chandata    -   2D array of channel settings
            %           globdata    -   1D array of global settings
            %
            %   EXAMPLE:   [n channels globals] = AMS1.ReadFlashBlock(2);
            %
            %   See also LoadFlashBlock, SaveActiveToFlashBlock 
         if nargin <2 || block <1 || block >5
            error('usage:ReadFlashBlock(blocknum). blocknum=1-5')
         else
            cdata=SendReceive(thisAMS,thisAMS.msgReadFlashBlock, block, 1);
            num=cdata(4);
            [chandata, globdata]=parseProgBlock(thisAMS,cdata(5:end-1));
         end
    end
    function success=LoadFlashBlock(thisAMS,block)
            %LoadFlashBlock - Make a saved setting set active.
            %
            %   LoadFlashBlock(n) Applies a saved settings to
            %                     the active instument controls.
            %
            %   SYNOPSIS: success = thisAMS.LoadFlashBlock(n)
            %
            %   INPUT: block - a number from 1 to 5 representing one of the
            %                  five complete instrument settings saved on
            %                  the instrument.
            %
            %   OUTPUT: success - Boolean value which indicates if the
            %                     operation was successful or not 
            %                           (1=TRUE, 0=FALSE).
            %
            %   EXAMPLE:   AMS1.LoadFlashBlock(2);
            %      
            %   See also ReadFlashBlock, SaveActiveToFlashBlock 
        if nargin <2 ||  block <1 || block >5
            error('You must put a block number from 1 to 5');
        else
            a=SendReceive(thisAMS,thisAMS.msgLoadFlashBlock, block, 1);
            if a(3)==194
                success=1;
                thisAMS.ProgramNumber=a(4);
              [thisAMS.priv_Channels, thisAMS.priv_Globals]=parseProgBlock(thisAMS,a(5:end-1));
              [bRef_2to8, bRef_10to16, bNotch_2to8, bNotch_10to16, ...
                  bRefNotch_1and9]=updateBitVars(thisAMS,thisAMS.priv_Channels);
            else
                success=0;
            end
        end
    end
    function success=SaveActiveToFlashBlock(thisAMS,block,name)
            %SaveActiveToFlashBlock - Takes active settings and saves in to
            %                         a flash slot.
            %
            %   SaveActiveToFlashBlock(n,'name') Takes active settings and
            %                      saves in to a flash slot identified by
            %                     the number n and the string 'name'.
            %
            %   SYNOPSIS: success = thisAMS.SaveActiveToFlashBlock(n,'aaa')
            %
            %   INPUT: block   - a number from 1 to 5 representing one 
            %                      of the five complete instrument settings 
            %                      saved on the instrument.
            %          name         name is the string that identifies 
            %                       the flash slot and number.
            %
            %   OUTPUT: success - Boolean value which indicates if the
            %                     operation was successful or not 
            %                           (1=TRUE, 0=FALSE).
            %
            %   EXAMPLE:   AMS1.SaveActiveToFlashBlock(2,'experiment');
            %        
            %   See also ReadFlashBlock, LoadFlashBlock 
            %
        if nargin <3 || block <1 || block >5
            success=0;
            error('You must put a block number and flash name');
        else
            bname=double(name)';
            obytes=SendReceive(thisAMS,thisAMS.msgSaveActiveProgramInFlash,[block; bname] , 1);
            success=isequal([obytes(4); obytes(41:end-2)],[block; bname]); 
        end
    end
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% 
end 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%% Verification and COM Methods Private Methods
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
methods(Access = private)   
    %% 
    function [ out1, out2, out3 ] = SendReceive(thisAMS, vrb, sendBytes, delay)
        %SendReceive - Sends a command to the instrument and receives the reply
        %
        %   [ out1, out2, out3 ] = SendReceive(command, data, delay) 
        %                          Communicates with the instrument to 
        %                          apply or query a setting.
        %
        %   SYNOPSIS: [bytes count string] = 
        %                       thisAMS.SendReceive(command, data, delay)
        %
        %   INPUT: vrb       -  The Verb message that tells the instrument 
        %                       what command to execute
        %          sendBytes -  The data used to complete the verb command
        %          delay     -  The delay to wait for a reply.
        %
        %   OUTPUT: out1 - The complete reply in binary array format
        %           out2 - The number of bytes in the binary array
        %           out3 - A string version of the output array
        %                      
        %
        %   EXAMPLE:   obytes
        %       = SendReceive(thisAMS,thisAMS.msgWriteSettingValue,bytes);
        %      
    out1=zeros(1);
    out2=zeros(1);
    out3=zeros(1);
%     s=serial(obj.amsPort,'BaudRate',115200,'DataBits',8,'StopBits', 1);
    s=thisAMS.PortInfo;
    % dataout=sscanf('A6 7F', '%x');
    terminate=sscanf('7F','%x');
    if nargin==4
        verb=double(vrb); 
    else
        delay=0.05;
    end
    if nargin==1 verb=hex2dec('A6'); sendBytes=[]; end
    if nargin==2,verb=double(vrb); sendBytes=[]; end
    if nargin==3, verb=double(vrb); end
    
    if iscolumn(sendBytes)
        dataout=[verb;sendBytes;terminate];
    else
        dataout=[verb;sendBytes';terminate];
    end
    fopen(s);
    fwrite(s,dataout);
    pause(delay);  % wait for transmission

    count=s.BytesAvailable;
    if count > 3
        recBytes=fread(s,count,'uint8');
        if nargout==1
            out1=recBytes;
        else
            out1=recBytes;
            out2=count;
        end
        if nargout==3
            strCell{1}='';
            recString='';
            cnt=1;
            for i=4:count-1
                if recBytes(i)==0
                    strCell{cnt}=recString;
                    recString='';
                    cnt=cnt+1;
                else
                    recString=[recString char(recBytes(i))];
                end
            end
            if cnt<3
                out3=cell2mat(strCell(1));
            else
                out3=strCell;
            end
            
        end
    end
    fclose(s);
    end
    %%
    function [errMSG, oBytes] = checkValue(obj,ch,set,val)
        %checkValue - Verifies value range and instrument data location.
        %             And returns the byte array format needed by the
        %             instrument.
        %
        %   [ errMSG outBytes ] = checkValue(channel, setting, value) 
        %                          Communicates with the instrument to 
        %                          apply or query a setting.
        %
        %   SYNOPSIS: [errMSG, oBytes] = 
        %                       thisAMS.checkValue(channel, setting, value)
        %
        %   INPUT: channel   -  The channel number to be changed.
        %          setting   -  The setting to be changed.
        %          value     -  The value to set.
        %
        %   OUTPUT: errMSG - Empty if no error, or a string.
        %           oBytes - The byte array to send to the instrument.
        %
        %   EXAMPLE:   obytes
        %       = SendReceive(thisAMS,thisAMS.msgWriteSettingValue,bytes);
        %         
    shift=zeros(size(ch));
    value=zeros(size(ch));
    oBytes=zeros(size(ch,1),2);
    errMSG='OK';
    for i=1:size(ch,1)
        switch set(i)
            case 2,  %mode
                if val(i)<0 || val(i)>2
                    errMSG='mode must be 0, 1 or 2';
                    return
                end
                shift(i)=48+ch(i)-1;
                value(i)=val(i);
            case 3, %HP
                if val(i)<0 || val(i)>7
                    errMSG='HP must be 0 to 7';
                    return
                end
                shift(i)= 0+ch(i)-1;
                value(i)=val(i);
            case 4 % Notch
                if val(i)<0 || val(i)>1
                    errMSG='Notch must be 0 or 1';
                    return
                end
                if ch(i)==1 || ch(i)==9
                    shift(i)=74;
                    if ch(i) == 1 %ch=1
                        if val(i)==1
                            value(i)=bitor(obj.bRefNotch_1and9, bitshift(1,4));
                        else
                            value(i)=bitand(obj.bRefNotch_1and9, bitcmp(bitshift(1,4),'uint8'));
                        end
                    else %ch=9
                        if val(i)==1
                            value(i)=bitor(obj.bRefNotch_1and9, bitshift(1,5));
                        else
                            value(i)=bitand(obj.bRefNotch_1and9, bitcmp(bitshift(1,5),'uint8'));
                        end
                    end
                    obj.bRefNotch_1and9=value(i);
                else
                    if ch(i)<9 %ch= 2-8
                        shift(i)=70;
                        if val(i)==1
                            value(i)=bitor(obj.bNotch_2to8, bitshift(1,ch(i)-1));
                        else
                            value(i)=bitand(obj.bNotch_2to8, bitcmp(bitshift(1,ch(i)-1),'uint8'));
                        end
                        obj.bNotch_2to8=value(i);
                        display(obj.bNotch_2to8)
                    else % ch= 10-16
                        shift(i)=71;
                        if val(i)==1
                            value(i)=bitor(obj.bNotch_10to16, bitshift(1,ch(i)-9));
                        else
                            value(i)=bitand(obj.bNotch_10to16, bitcmp(bitshift(1,ch(i)-9),'uint8'));
                        end
                         obj.bNotch_10to16=value(i);
                    end
                end
            case 5, % LP
                if val(i)<0 || val(i)>7
                    errMSG='LP must be 0 to 7';
                    return
                end
                shift(i)=16+ch(i)-1; 
                value(i)=val(i);
            case 6, %Gain
                if val(i)<0 || val(i)>12
                    errMSG='Gain must be 0 to 12';
                    return
                end
                shift(i)=32+ch(i)-1; 
                value(i)=val(i);
            case 7 %Ref
                if val(i)<0 || val(i)>1
                    errMSG='Reference must be 0 or 1';
                    return
                end
                if ch(i)==1 || ch(i)==9
                    shift(i)=74;
                    if ch(i) == 1 %ch=1
                        if val(i)==1
                            value(i)=bitor(obj.bRefNotch_1and9, bitshift(1,6));
                        else
                            value(i)=bitand(obj.bRefNotch_1and9, bitcmp(bitshift(1,6),'uint8'));
                        end
                    else %ch=9
                        if val(i)==1
                            value(i)=bitor(obj.bRefNotch_1and9, bitshift(1,7));
                        else
                            value(i)=bitand(obj.bRefNotch_1and9, bitcmp(bitshift(1,7),'uint8'));
                        end
                    end
                    obj.bRefNotch_1and9=value(i);
                else
                    if ch(i)<9 %ch= 2-8
                        shift(i)=68;
                        if val(i)==1
                            value(i)=bitor(obj.bRef_2to8, bitshift(1,ch(i)-1));
                        else
                            value(i)=bitand(obj.bRef_2to8, bitcmp(bitshift(1,ch(i)-1),'uint8'));
                        end
                        obj.bRef_2to8=value(i);
                    else % ch= 10-16
                        shift(i)=69;
                        if val(i)==1
                            value(i)=bitor(obj.bRef_10to16, bitshift(1,ch(i)-9));
                        else
                            value(i)=bitand(obj.bRef_10to16, bitcmp(bitshift(1,ch(i)-9),'uint8'));
                        end
                         obj.bRef_10to16=value(i);
                    end
                end
        end
       oBytes(i,1)=shift(i);
       oBytes(i,2)=value(i);
    end 
    end
    %%
    function success = WriteValue(thisAMS, bytes)
        %WriteValue - Writes a value, and verifies the return.
        %
        %   SYNOPSIS:  WriteValue(thisAMS, outBytes(i,:)')
        %
        %   INPUT: bytes   -  bytes to send to the instrument.
        %
        %   OUTPUT: success - Boolean value which indicates if the
        %                     operation was successful or not 
        %                           (1=TRUE, 0=FALSE).
        %
        %   EXAMPLE:  WriteValue(thisAMS,bytes);
        %   
       obytes=SendReceive(thisAMS,thisAMS.msgWriteSettingValue,bytes);
       success=isequal(obytes(4:end-1),bytes); 
    end   
    %%
    function [Chdata GlbData]=parseProgBlock(~,indata)
        %parseProgBlock - Converts binary byte array into 2D channels array
        %
        %   SYNOPSIS:  WriteValue(thisAMS, outBytes(i,:)')
        %
        %   INPUT: bytes   -  bytes to send to the instrument.
        %
        %   OUTPUT: Chdata  - 2D array of instrument settings formated as:
        %                       [Ch# Mode HP Notch LP Gain Ref]
        %           GlbData - 1D array of global instrument settings:
        %                    [StimSource CalAmp CalOn RefSource MonA MonB]
        %
        %   EXAMPLE:  WriteValue(thisAMS,bytes);
        %   
        temp=zeros(16,7,'uint8');
        temp2=[0 0 0 0 0 0 ];
        for ch=1:16
            b1=ch*2-1;
            b2=ch*2;
            temp(ch,1)=ch;
            temp(ch,2)=bitshift(bitand(indata(b2),96),-5);%mode  0x60
            temp(ch,3)=bitshift(bitand(indata(b1),112),-4);%HP 0x70
            temp(ch,4)=bitshift(bitand(indata(b1),128),-7);%Notch 0x80
            temp(ch,5)=bitshift(bitand(indata(b1),14),-1);%LP 0x0E
            temp(ch,6)=bitshift(bitand(indata(b2),30),-1);%Gain 0x1E
            temp(ch,7)=bitshift(bitand(indata(b2),128),-7);%Ref 0x80
        end
        %lblGlobalColums='[StimSource CalAmp CalOn RefSource MonA MonB]';
        temp2(1)=bitshift(bitand(indata(35),128),-7);%Stim 0x80
        temp2(2)=bitshift(bitand(indata(35),24),-3);%CalAmp 0x18
        temp2(3)=bitshift(bitand(indata(35),2),-1);%CalOn 0x02
        if size(indata,1)==36 
            temp2(4)=indata(36)+1;%RefSource for Model 3600
        else
            temp2(4)=bitshift(bitand(indata(35),64),-6);%Bus 0x40 for Model 3500
        end
        temp2(5)=indata(33)+1; %monA
        temp2(6)=indata(34)+1; %monB
        if nargout==1
             Chdata=temp;
        else
            Chdata=temp;
            GlbData=temp2;
        end          
    end
    %%
    function [bRef28,bRef1016,bNotch28,bNotch1016,bRefNotch19]=updateBitVars(~,temp)
        %Updates private bit formated variables 
        bRef28=temp(2,7)*2+temp(3,7)*4+temp(4,7)*8+ ...
               temp(5,7)*16+temp(6,7)*32+temp(7,7)*64+temp(8,7)*128;
        bRef1016=temp(10,7)*2+temp(11,7)*4+temp(12,7)*8+ ...
                 temp(13,7)*16+temp(14,7)*32+temp(15,7)*64+temp(16,7)*128;
        bNotch28=temp(2,4)*2+temp(3,4)*4+temp(4,4)*8+ ...
                 temp(5,4)*16+temp(6,4)*32+temp(7,4)*64+temp(8,4)*128;
        bNotch1016=temp(10,4)*2+temp(11,4)*4+temp(12,4)*8+ ...
                 temp(13,4)*16+temp(14,4)*32+temp(15,4)*64+temp(16,4)*128;
        bRefNotch19=temp(1,7)*4+temp(9,7)*8+ ...
                    temp(1,4)*16+temp(9,4)*32;
    end 
    %%
    function [errMSG, oBytes] = checkGlobes(obj,set,val)
        %'[StimSource CalAmp CalOn RefSource MonA MonB]';
        %     35        35    35      35or36  33   34
        shift=zeros(size(set));
        value=zeros(size(set));
        oBytes=zeros(size(set,2),2);
        errMSG='OK';
        for i=1:size(set,2)
            switch set(i)
                case 1 %Stim
                    if val(i)<0 || val(i)>1
                        errMSG='StimSource must be 0 or 1';
                    return
                    end
                    shift(i)=72;
                    value(i)=val(i);               
                case 2 %CalAmp
                     if val(i)<0 || val(i)>3
                        errMSG='Calibration Amp must be 0 to 3';
                    return
                    end
                    shift(i)=66;
                    value(i)=val(i);    
                case 3 %CalOn
                     if val(i)<0 || val(i)>1
                        errMSG='StimSource must be 0 or 1';
                    return
                    end
                    shift(i)=73;
                    value(i)=val(i);                     
                case 4 %refsourse
                     if val(i)<0 || val(i)>17
                        errMSG='Reference must be 0 to 17';
                    return
                    end
                    shift(i)=67;
                    value(i)=val(i)-1;                      
                case 5 %mona
                     if val(i)<1 || val(i)>16
                        errMSG='Monitor must be 1 to 16';
                    return
                    end
                    shift(i)=64;
                    value(i)=val(i)-1;   
                case 6 %monb
                    if val(i)<1 || val(i)>16
                        errMSG='Monitor must be 1 to 16';
                    return
                    end
                    shift(i)=65;
                    value(i)=val(i)-1; 
            end
        oBytes(i,1)=shift(i);
        oBytes(i,2)=value(i); 
        end
    end
    %%
    function outdata= convertdata(~,indata, globs)
        %convertdata - Converts 2D channels array into binary byte array
        %
        %   SYNOPSIS:  convertdata(thisAMS, indata, globs)
        %
        %   INPUT: indata   -  2D channel settings array
        %          globs    -  1D global settings array
        %
        %   OUTPUT: outdata   - instrument byte array
        %   
        % ChannelColumns = '[Ch# Mode HP Notch LP Gain Ref]';
        % globs='[StimSource CalAmp CalOn RefSource MonA MonB]';
        temp=zeros(36,1,'uint8');
        for ch=1:16
            temp(ch*2-1)=indata(ch,5)*2+indata(ch,3)*16+indata(ch,4)*128;
            temp(ch*2)=indata(ch,6)*2+indata(ch,2)*32+indata(ch,7)*128;
        end
        if globs(4)<2
            rf=globs(4);
        else 
            rf=0;
        end
        temp(33)=globs(5)-1;
        temp(34)=globs(6)-1;
        temp(35)=globs(3)*2+globs(2)*8+rf*64+globs(1)*128;
        temp(36)=globs(4);
        outdata=temp;
    end
end %methods private
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%    
end %class








